from enum import Enum


class MaidrKey(Enum):
    """
    Enumeration class representing keys used in the Maidr application.
    """

    AXES = "axes"
    CAPTION = "caption"
    DATA = "data"
    FILL = "fill"
    LABEL = "label"
    LEVEL = "level"
    ID = "id"
    ORIENTATION = "orientation"
    SELECTOR = "selector"
    SUBTITLE = "subtitle"
    TITLE = "title"
    TYPE = "type"
    X = "x"
    Y = "y"
